| <!DOCTYPE html><html lang="en"><head><meta charset="utf-8"><meta name="viewport" content="width=device-width, initial-scale=1.0"><meta name="generator" content="rustdoc"><meta name="description" content="API documentation for the Rust `RegionKind` type in crate `hir_ty`."><title>RegionKind in hir_ty::next_solver - Rust</title><script>if(window.location.protocol!=="file:")document.head.insertAdjacentHTML("beforeend","SourceSerif4-Regular-6b053e98.ttf.woff2,FiraSans-Italic-81dc35de.woff2,FiraSans-Regular-0fe48ade.woff2,FiraSans-MediumItalic-ccf7e434.woff2,FiraSans-Medium-e1aa3f0a.woff2,SourceCodePro-Regular-8badfe75.ttf.woff2,SourceCodePro-Semibold-aa29a496.ttf.woff2".split(",").map(f=>`<link rel="preload" as="font" type="font/woff2"href="../../static.files/${f}">`).join(""))</script><link rel="stylesheet" href="../../static.files/normalize-9960930a.css"><link rel="stylesheet" href="../../static.files/rustdoc-e56847b5.css"><meta name="rustdoc-vars" data-root-path="../../" data-static-root-path="../../static.files/" data-current-crate="hir_ty" data-themes="" data-resource-suffix="" data-rustdoc-version="1.91.1 (ed61e7d7e 2025-11-07)" data-channel="1.91.1" data-search-js="search-e256b49e.js" data-stringdex-js="stringdex-c3e638e9.js" data-settings-js="settings-c38705f0.js" ><script src="../../static.files/storage-e2aeef58.js"></script><script defer src="sidebar-items.js"></script><script defer src="../../static.files/main-6dc2a7f3.js"></script><noscript><link rel="stylesheet" href="../../static.files/noscript-263c88ec.css"></noscript><link rel="alternate icon" type="image/png" href="../../static.files/favicon-32x32-eab170b8.png"><link rel="icon" type="image/svg+xml" href="../../static.files/favicon-044be391.svg"></head><body class="rustdoc type"><!--[if lte IE 11]><div class="warning">This old browser is unsupported and will most likely display funky things.</div><![endif]--><rustdoc-topbar><h2><a href="#">RegionKind</a></h2></rustdoc-topbar><nav class="sidebar"><div class="sidebar-crate"><h2><a href="../../hir_ty/index.html">hir_ty</a><span class="version">0.0.0</span></h2></div><div class="sidebar-elems"><section id="rustdoc-toc"><h2 class="location"><a href="#">Region<wbr>Kind</a></h2><h3><a href="#aliased-type">Aliased Type</a></h3><h3><a href="#variants">Variants</a></h3><ul class="block variant"><li><a href="#variant.ReBound" title="ReBound">ReBound</a></li><li><a href="#variant.ReEarlyParam" title="ReEarlyParam">ReEarlyParam</a></li><li><a href="#variant.ReErased" title="ReErased">ReErased</a></li><li><a href="#variant.ReError" title="ReError">ReError</a></li><li><a href="#variant.ReLateParam" title="ReLateParam">ReLateParam</a></li><li><a href="#variant.RePlaceholder" title="RePlaceholder">RePlaceholder</a></li><li><a href="#variant.ReStatic" title="ReStatic">ReStatic</a></li><li><a href="#variant.ReVar" title="ReVar">ReVar</a></li></ul></section><div id="rustdoc-modnav"><h2><a href="index.html">In hir_<wbr>ty::<wbr>next_<wbr>solver</a></h2></div></div></nav><div class="sidebar-resizer" title="Drag to resize sidebar"></div><main><div class="width-limiter"><section id="main-content" class="content"><div class="main-heading"><div class="rustdoc-breadcrumbs"><a href="../index.html">hir_ty</a>::<wbr><a href="index.html">next_solver</a></div><h1>Type Alias <span class="type">Region<wbr>Kind</span> <button id="copy-path" title="Copy item path to clipboard">Copy item path</button></h1><rustdoc-toolbar></rustdoc-toolbar><span class="sub-heading"><a class="src" href="../../src/hir_ty/next_solver/region.rs.html#19">Source</a> </span></div><pre class="rust item-decl"><code>pub type RegionKind<'db> = RegionKind<<a class="struct" href="interner/struct.DbInterner.html" title="struct hir_ty::next_solver::interner::DbInterner">DbInterner</a><'db>>;</code></pre><h2 id="aliased-type" class="section-header">Aliased Type<a href="#aliased-type" class="anchor"></a></h2><pre class="rust item-decl"><code>pub enum RegionKind<'db> { |
| ReEarlyParam(<a class="struct" href="struct.EarlyParamRegion.html" title="struct hir_ty::next_solver::EarlyParamRegion">EarlyParamRegion</a>), |
| ReBound(BoundVarIndexKind, <a class="struct" href="struct.BoundRegion.html" title="struct hir_ty::next_solver::BoundRegion">BoundRegion</a>), |
| ReLateParam(<a class="struct" href="struct.LateParamRegion.html" title="struct hir_ty::next_solver::LateParamRegion">LateParamRegion</a>), |
| ReStatic, |
| ReVar(RegionVid), |
| RePlaceholder(<a class="struct" href="interner/struct.Placeholder.html" title="struct hir_ty::next_solver::interner::Placeholder">Placeholder</a><<a class="struct" href="struct.BoundRegion.html" title="struct hir_ty::next_solver::BoundRegion">BoundRegion</a>>), |
| ReErased, |
| ReError(<a class="struct" href="struct.ErrorGuaranteed.html" title="struct hir_ty::next_solver::ErrorGuaranteed">ErrorGuaranteed</a>), |
| }</code></pre><h2 id="variants" class="variants section-header">Variants<a href="#variants" class="anchor"></a></h2><div class="variants"><section id="variant.ReEarlyParam" class="variant"><a href="#variant.ReEarlyParam" class="anchor"></a><h3 class="code-header">ReEarlyParam(<a class="struct" href="struct.EarlyParamRegion.html" title="struct hir_ty::next_solver::EarlyParamRegion">EarlyParamRegion</a>)</h3></section><div class="docblock"><p>A region parameter; for example <code>'a</code> in <code>impl<'a> Trait for &'a ()</code>.</p> |
| <p>There are some important differences between region and type parameters. |
| Not all region parameters in the source are represented via <code>ReEarlyParam</code>: |
| late-bound function parameters are instead lowered to a <code>ReBound</code>. Late-bound |
| regions get eagerly replaced with <code>ReLateParam</code> which behaves in the same way as |
| <code>ReEarlyParam</code>. Region parameters are also sometimes implicit, |
| e.g. in <code>impl Trait for &()</code>.</p> |
| </div><section id="variant.ReBound" class="variant"><a href="#variant.ReBound" class="anchor"></a><h3 class="code-header">ReBound(BoundVarIndexKind, <a class="struct" href="struct.BoundRegion.html" title="struct hir_ty::next_solver::BoundRegion">BoundRegion</a>)</h3></section><div class="docblock"><p>A higher-ranked region. These represent either late-bound function parameters |
| or bound variables from a <code>for<'a></code>-binder.</p> |
| <p>While inside of a function, e.g. during typeck, the late-bound function parameters |
| can be converted to <code>ReLateParam</code> by calling <code>tcx.liberate_late_bound_regions</code>.</p> |
| <p>Bound regions inside of types <strong>must not</strong> be erased, as they impact trait |
| selection and the <code>TypeId</code> of that type. <code>for<'a> fn(&'a ())</code> and |
| <code>fn(&'static ())</code> are different types and have to be treated as such.</p> |
| </div><section id="variant.ReLateParam" class="variant"><a href="#variant.ReLateParam" class="anchor"></a><h3 class="code-header">ReLateParam(<a class="struct" href="struct.LateParamRegion.html" title="struct hir_ty::next_solver::LateParamRegion">LateParamRegion</a>)</h3></section><div class="docblock"><p>Late-bound function parameters are represented using a <code>ReBound</code>. When |
| inside of a function, we convert these bound variables to placeholder |
| parameters via <code>tcx.liberate_late_bound_regions</code>. They are then treated |
| the same way as <code>ReEarlyParam</code> while inside of the function.</p> |
| <p>See <a href="https://rustc-dev-guide.rust-lang.org/early_late_parameters.html">https://rustc-dev-guide.rust-lang.org/early_late_parameters.html</a> for |
| more info about early and late bound lifetime parameters.</p> |
| </div><section id="variant.ReStatic" class="variant"><a href="#variant.ReStatic" class="anchor"></a><h3 class="code-header">ReStatic</h3></section><div class="docblock"><p>Static data that has an “infinite” lifetime. Top in the region lattice.</p> |
| </div><section id="variant.ReVar" class="variant"><a href="#variant.ReVar" class="anchor"></a><h3 class="code-header">ReVar(RegionVid)</h3></section><div class="docblock"><p>A region variable. Should not exist outside of type inference.</p> |
| </div><section id="variant.RePlaceholder" class="variant"><a href="#variant.RePlaceholder" class="anchor"></a><h3 class="code-header">RePlaceholder(<a class="struct" href="interner/struct.Placeholder.html" title="struct hir_ty::next_solver::interner::Placeholder">Placeholder</a><<a class="struct" href="struct.BoundRegion.html" title="struct hir_ty::next_solver::BoundRegion">BoundRegion</a>>)</h3></section><div class="docblock"><p>A placeholder region – the higher-ranked version of <code>ReLateParam</code>. |
| Should not exist outside of type inference.</p> |
| <p>Used when instantiating a <code>forall</code> binder via <code>infcx.enter_forall</code>.</p> |
| </div><section id="variant.ReErased" class="variant"><a href="#variant.ReErased" class="anchor"></a><h3 class="code-header">ReErased</h3></section><div class="docblock"><p>Erased region, used by trait selection, in MIR and during codegen.</p> |
| </div><section id="variant.ReError" class="variant"><a href="#variant.ReError" class="anchor"></a><h3 class="code-header">ReError(<a class="struct" href="struct.ErrorGuaranteed.html" title="struct hir_ty::next_solver::ErrorGuaranteed">ErrorGuaranteed</a>)</h3></section><div class="docblock"><p>A region that resulted from some other error. Used exclusively for diagnostics.</p> |
| </div></div><script src="../../type.impl/ra_ap_rustc_type_ir/region_kind/enum.RegionKind.js" data-self-path="hir_ty::next_solver::region::RegionKind" async></script></section></div></main></body></html> |